/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Image;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JComboBox;

import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * @author Robert Jan van der Waals
 * @version 1.9
 * @since 1.4
 * 
 * @deprecated performance is too low!
 */
public class IconSelector extends JComboBox implements ISettingsComponent {
    
    private static final Collection images = new ArrayList();
    private Image img;
    private URL url;
    
    public IconSelector() {
        super();
        getImages();
        fill();
    }
    
    private void fill() {
        removeAllItems();
        
        if (url != null) {
            addItem(new Item(url.getFile(), img));
        }
        for (Iterator iter = images.iterator(); iter.hasNext(); ) {
            addItem(iter.next());
        }
    }
     
    private void getImages() {
        if (images.size() == 0) {
            String baseDir = "icons_user";
            File dir = new File(baseDir);
            String[] files = dir.list();
    
            for (int i = 0; i < files.length; i++) {
                if (i > 100) {
                    break;
                }
                
                try {
                    String file = "file://" + dir.getCanonicalPath() + File.separator + files[i];
                    URL imageUrl = new URL(file);
                    ImageIcon icon = new ImageIcon(imageUrl);
                    images.add(new Item(file, icon.getImage()));
                } catch (Exception exp) {
                    Messages.add(exp);
                }
            }
        }
    }
    
    /**
     * Returns the image as a BASE64 string
     */
    public Object getValue() {
        Item item = (Item) getSelectedItem();
        if (item != null) {
            try {
                URL imageUrl = new URL("file://" + item.getFilename());
                return Utilities.fileToBase64String(imageUrl);
            } catch (Exception exp) {
                Messages.add(exp);
            }
        } 
        return null;
    }
    
    public void setValue(Object o) {
        if (o instanceof URL) {
            url = (URL) o;
            img = getImage(url);
        } else {
            String s = o.toString();
            if (s.endsWith("jpg")) {
                try {
                    if (!s.startsWith("file:")) {
                        url = new URL("file:" + s);
                    } else {
                        url = new URL(s);
                    }
                } catch (Exception exp) {
                    Messages.add(
                            Resources.getText(DCMessages.msgCouldNotLoadPicture), Messages._ERROR);
                    Messages.add(exp);
                }
            } else if (s.length() > 1) {
                url = Utilities.createCachedImage(s);
            } 
            img = getImage(url);
        }
        fill();
    }
    
    /**
     * Returns the java.awt.Image
     */
    private Image getImage(URL url) {
        try {
            if (url != null) {
                ImageIcon icon = new ImageIcon(url);
                return icon.getImage();
            } else {
                return null;
            }
        } catch (Exception exp) {
            Messages.add(exp);
            return null;
        }        
    }
    
    private class Item {
        private final String filename;
        private final Image icon;
        
        public Item(String filename, Image icon) {
            this.filename = filename;
            this.icon = icon;
        }
        
        public Image getIcon() {
            return icon;
        }
        
        public String getFilename() {
            return filename;
        }
        
        public String toString() {
            return "";
        }
    }
}
