/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.DCMenuBar;
import net.sf.dc.console.book.BookCategoryForm;
import net.sf.dc.console.book.InsertBookPanel;
import net.sf.dc.console.book.SearchBookPanel;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.da.ImportCDDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Book module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class BookModule extends DCModule {
    
    private final DCMenuBar menuBar;
    
    /**
     * Initializes this module
     */
    public BookModule() {
        super(DCModules._BOOK,
              Resources.getText(DCLabels.lblModuleBook),
              "books",
              "boo",
              "books",
              false,
              true,
              false);
        menuBar = new DCMenuBar(this);
    }

    public DCMenuBar getMenuBar() {
        return menuBar;
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoBookModule;
    }
    
    public String getObjectName() {
        return Resources.getText(DCLabels.lblBook);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(Resources.getText(DCLabels.lblTools));

        JMenuItem menuCat = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuCategory16,
                                                Resources.getText(DCLabels.lblCatagoryForm));
        menuCat.addActionListener(new CategoryFormMenuAction());
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                Resources.getText(DCLabels.lblSoftwareImport));
        menuImport.addActionListener(new ImportCDAction());
        menu.add(menuCat);
        menu.addSeparator();
        menu.add(menuImport);
        
        return menu;
    }      
        
    /**
     * Return the object managed by this module 
     * @see Book
     */
    public DCObject getDCObject() {
        return new Book();
    }
        
    /**
     * Create the insert and search panel
     */
    protected void initializeUI() {
        insertPanel = new InsertBookPanel();
        searchPanel = new SearchBookPanel();
    }
        
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsBook;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stBookRequiredFields;
    }
    
    public void startImport() {
new MessageBox("Not implemented (yet) ! Sorry for the inconvenience. We suggest using the online search instead!", MessageBox._INFORMATION);
    }
    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
							     			  String nameOfCD) {
        return null;
    }
    
    /**
     * Initalizes all the fields
     */
    protected void initializeFields() {
        fields = new DCFieldDefinition[12];
        fields[Book._ID] = new DCFieldDefinition(Resources.getText(DCLabels.lblID),
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[Book._A_TITLE] = new DCFieldDefinition(Resources.getText(DCLabels.lblTitle),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[Book._B_DESCRIPTION] = new DCFieldDefinition(Resources.getText(DCLabels.lblDescription),
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");
        fields[Book._C_PUBLISHER] = new DCFieldDefinition(Resources.getText(DCLabels.lblPublisher),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Publisher");                                       
        fields[Book._D_AUTHOR] = new DCFieldDefinition(Resources.getText(DCLabels.lblAuthor),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Author");                                     
        fields[Book._E_RELEASEYEAR] = new DCFieldDefinition(Resources.getText(DCLabels.lblReleaseYear),
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "ReleaseYear");   
        fields[Book._F_WEBPAGE] = new DCFieldDefinition(Resources.getText(DCLabels.lblWebPage),
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._URLFIELD,
                                        Repository.datatypes._STRING,
                                        "Webpage");   
        fields[Book._G_CATEGORY] = new DCFieldDefinition(Resources.getText(DCLabels.lblBookCategory),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._BOOKCATEGORYCOMBO,
                                        Repository.datatypes._TYPEREFERENCE,
                                        "Type");  
        fields[Book._H_ISBN] = new DCFieldDefinition(Resources.getText(DCLabels.lblISBN),
                                        false,
                                        true,
                                        false,
                                        20,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ISBN");          
        fields[Book._I_PICTUREFRONT] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureFront),
                                        false,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");       
        fields[Book._J_LOCATION] = new DCFieldDefinition(Resources.getText(DCLabels.lblLocation),
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");     
        fields[Book._K_RATING] = new DCFieldDefinition(Resources.getText(DCLabels.lblRating),
                                        false,
                                        true,
                                        false,
                                        10,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating"); 
    }
    
    public DCObject[] getDefaultData() {
        return new DCObject[0];
    }
    
    
    private class CategoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            BookCategoryModule bookCatMod = 
                (BookCategoryModule) DCModules.getModule(DCModules._BOOKCAT);
            BookCategoryForm categoryForm = bookCatMod.getForm(); 
            categoryForm.setVisible(true);
        }
    }        
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }      
}
