/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.jflexsett.settings;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * A settings group contains specific settings. A group can have
 * on child, creating a two level settings hierarchy. Deeper 
 * hierarchies are not supported as yet.
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class SettingsGroup {
    
    private String helpIndex = "";
    private String sKey = "";
    private LinkedHashMap htSettings = new LinkedHashMap();
    private Hashtable htChildren = new Hashtable();
    
    /**
     * Creates a new settings group
     * 
     * @param sKey a unique identifier of this group
     */
    public SettingsGroup(String sKey, String helpIndex) {
        this.sKey = sKey;
        this.helpIndex = helpIndex;
    }

    /**
     * Adds a child to the settings group
     */
    public void addChild(SettingsGroup child) {
        htChildren.put(child.getKey(), child);
    }
    
    public String getKey() {
        return sKey;   
    }
    
    public String getHelpIndex() {
    	return helpIndex;
    }
    
    public Hashtable getChildren() {
        return htChildren;
    }
    
    public SettingsGroup getChild(String sGroupKey) {
        return (SettingsGroup) htChildren.get(sGroupKey);
    }
    
    public Map getSettings() {
        return htSettings;
    }
    
    protected Setting getSetting(String sKey) {
        Setting setting = (Setting) htSettings.get(sKey);
        
        if (setting != null) {
            return setting; 
        } 
        
        for (Enumeration enumerator = htChildren.elements(); enumerator.hasMoreElements(); ) {
            SettingsGroup settingsGroup = (SettingsGroup) enumerator.nextElement();
            if (settingsGroup != null) {
                setting = settingsGroup.getSetting(sKey);
                if (setting != null) {
                    break;   
                }
            }
        }
        return setting;
    }
    
    protected void add(Setting setting) {
        htSettings.put(setting.getKey(), setting);
    }
    
    public String toString() {
        return sKey; 
    }
}
