/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Properties;

/**
 * Creates a java source file (resource file) from a property file
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ResourceBuilder {

    private static BufferedWriter bw = null;
    private static String NL = "\n";
    private static final String sPackage = "package net.sf.dc.resources;";


    public static void main(String[] args) {
        try {
            if(args.length < 3) {
                showUsage();
            }

            File   src  = new File(args[0]);
            File   dest = new File(args[1]);
            String modulePrefix    = args[2];
            String fullQualClsName = args[3];
            String clsName = null;
            File   subDirs = null;
            String dottedPath = null;
    
            //Determine the basename of the class
            int dot = fullQualClsName.lastIndexOf(".");
            if(dot == -1) {
                clsName = fullQualClsName;
            } else {
                clsName = fullQualClsName.substring(dot+1);
                dottedPath = fullQualClsName.substring(0,dot);
                subDirs = new File(dottedPath.replace('.', File.separatorChar));
            }   

            //Handle the src dir/file
            if(!src.exists()) {
                throw new RuntimeException("The src dir: '"+src+"' does not exists");
            }

            File srcFile = src;
            if(subDirs != null) {
                srcFile = new File(src, subDirs.getPath());
            }

            srcFile = new File(srcFile, clsName+".properties");
            if(!srcFile.exists()) {
                throw new RuntimeException("The src file: '"+srcFile+"' does not exists");
            }

            //Handle the dest dir
            if(!dest.exists()) {
                throw new RuntimeException("The dest dir: '"+dest+"' does not exists");
            }

            File destDir = dest;
            if(subDirs != null) {
                destDir = new File(dest, subDirs.getPath());
            }

            if(!destDir.exists()) {
                throw new RuntimeException("TODO: recursively create destDir");
            }
            File destFile = new File(destDir, clsName+".java");
    
            Properties prop = new Properties();
            prop.load(new FileInputStream(srcFile));
    
            bw = new BufferedWriter(new FileWriter(destFile));
    
            writeJavaClassDecl(modulePrefix, clsName);
    
            for(Iterator keys = prop.keySet().iterator();keys.hasNext();) {
                writeJavaConstants(modulePrefix, (String)keys.next());
            }
    
            writeNL();
    
            writeJavaContentDecl();
            for(Iterator keys = prop.keySet().iterator();keys.hasNext();) {
                String key = (String)keys.next();
                writeJavaContentKV(key, (String) prop.get(key));
            }
            writeJavaContentEnd();
            writeJavaClassEnd();
    
            bw.close();
    
            System.exit(0);
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }

    private static void writeJavaClassDecl(String prefix, String clsName) throws IOException {
        bw.write("//Auto generated file"+NL);
        bw.write(sPackage);
        bw.write(NL);
        bw.write("import net.sf.ext.util.ModularResourceBundle;"+NL);
        bw.write(NL);
        bw.write("public class "+clsName+ " extends ModularResourceBundle {"+NL);
        bw.write(NL);
        bw.write("    public Object[][] getContents() {"+NL);
        bw.write("        return contents;"+NL);
        bw.write("    }"+NL);
        bw.write(NL);
        bw.write("    public String getAcronym() {"+NL);
        bw.write("        return \""+prefix+"\";"+NL);
        bw.write("    }"+NL);
        bw.write(NL);
    }

    private static void writeNL() throws IOException {
        bw.write(NL);
    }

    private static void writeJavaConstants(String prefix, String key) throws IOException {
        bw.write("    public static final String "+key+" = \""+prefix+"."+key+"\";"+NL);
    }

    private static void writeJavaContentDecl() throws IOException {
        bw.write("    static final Object[][] contents = {"+NL);
    }

    private static void writeJavaContentKV(String key, String value) throws IOException {
        bw.write("        {"+key+", \""+value+"\"},"+NL);
    }

    private static void writeJavaContentEnd() throws IOException {
        bw.write("    };"+NL);
    }

    private static void writeJavaClassEnd() throws IOException {
        bw.write("}"+NL);
    }

    private static void showUsage() {
        System.out.println("The following syntax is required:\n"
            + "  source directory \n destination directory \n Acronym \n Fully Qualified className\n");
        System.exit(1);
    }
}
